/*

Mike DeFehr May 2013

*/

USE tempdb
GO
IF NOT OBJECT_ID('OrderGet') IS NULL DROP PROC OrderGet
GO
CREATE PROC OrderGet
@OrderID int
AS
BEGIN
	SELECT	OrderDate,
			CustomerName
	FROM Orders
	WHERE OrderID = @OrderID;

	SELECT	OrderDetailID, 
			OrderID,
			ItemName,
			OrderQty,
			ItemPrice
	FROM OrderDetails
	WHERE OrderID = @OrderID
END
GO

IF OBJECT_ID('OrderUpdate') IS NOT NULL DROP PROC OrderUpdate
IF EXISTS(SELECT * FROM sys.types WHERE name = 'OrderDetails_type') DROP TYPE OrderDetails_type
GO
CREATE TYPE OrderDetails_type AS TABLE 
(	OrderDetailID	int,
	OrderID			int,
	ItemName		varchar(255),
	OrderQty		decimal(19,4),
	ItemPrice		money
)
GO

CREATE PROC OrderUpdate
@OrderID int,
@OrderDate date,
@CustomerName VARCHAR(255),
@OD dbo.OrderDetails_type READONLY
AS
BEGIN
	UPDATE Orders SET 
		OrderDate = @OrderDate, 
		CustomerName = @CustomerName 
	WHERE OrderID = @OrderID;


	MERGE INTO OrderDetails tgt
	USING @OD src
	ON tgt.OrderDetailID = src.OrderDetailID
	WHEN MATCHED AND (	tgt.ItemName <> src.ItemName
					OR	tgt.OrderQty <> src.OrderQty
					OR	tgt.ItemPrice <> src.ItemPrice	) THEN 
	UPDATE
	SET	ItemName = src.ItemName,
		OrderQty = src.OrderQty,
		ItemPrice = src.ItemPrice
	WHEN NOT MATCHED BY TARGET THEN
	INSERT	(	OrderID,
				ItemName,
				OrderQty,
				ItemPrice
			)
	VALUES	(	src.OrderID,
				src.ItemName,
				src.OrderQty,
				src.ItemPrice
			)
	WHEN NOT MATCHED BY SOURCE AND tgt.OrderID = @OrderID 
		THEN DELETE
	OUTPUT	$Action AS ChangeType,
			COALESCE(src.OrderDetailID,deleted.OrderDetailID,inserted.OrderDetailID) AS OrderDetailID,
			COALESCE(src.OrderID,deleted.OrderID) AS OrderID,
			COALESCE(src.ItemName,deleted.ItemName) AS ItemName,
			COALESCE(src.OrderQty,deleted.OrderQty) AS OrderQty,
			COALESCE(src.ItemPrice,deleted.ItemPrice) AS ItemPrice;
END
GO		
